#include <Servo.h>
#include <EEPROM.h>
#include <Wire.h>

#include "ROBO3_UltrasonicSensor.h"
#include "ROBO3_LedMatrix.h"
#include "ROBO3_RGBled.h"
#include "ROBO3_DCMotorTC118S.h"
#include "ROBO3_OTPvoice.h"

ROBO3_UltrasonicSensor us;
ROBO3_LedMatrix ledmatrix;
ROBO3_DCMotorTC118S motor1, motor2;
ROBO3_OTPvoice voice;
ROBO3_RGBled leds;

uint16_t hist[4] = { 0 };

void setup()
{
    ledmatrix.setpin_mio(1);
    leds.setpin_mio(onBoardPort);
    us.setpin_mio(2);
    motor1.setpin_mio(MIO_M1);
    motor2.setpin_mio(MIO_M2);
    voice.setpin_mio(onBoardPort);
}

// Filtering the input values with a rolling buffer of size 4 and getting the mean value
uint16_t new_value(uint16_t v) {
  hist[0] = hist[1];
  hist[1] = hist[2];
  hist[2] = hist[3];
  hist[3] = v;

  // max value is 300 so no overflow possible
  uint16_t sum = hist[0] + hist[1] + hist[2] + hist[3];
  return sum / 4;
}

void loop()
{
    uint16_t filtered = new_value(us.read());
    ledmatrix.showNumber(filtered,4);
    
    if (filtered < 20) {
      uint8_t red = 255;
      uint8_t green = 0;
      uint8_t blue = 0;
      leds.setColor(0, red, green, blue); //set RGB value to all LEDs
      leds.show();  // send updated values
      voice.write(MUSIC_SUPRISED);
      delay(150);
    
      motor1.run(-75);
      motor2.run(-75);
      delay(500);
      
      red  = 0; green = 255; blue = 0;
      leds.setColor(0, red, green, blue);
      leds.show();
      voice.write(MUSIC_STOP);
      motor1.run(0);
      motor2.run(0);
      delay(250);
      
      leds.setColor(0, 0, 0, 0);
      leds.show();
    }
    delay(50);
}
